import sys
import shutil
import subprocess
from os.path import basename, isdir, islink, isfile, join


NO_EXT = (
    '.py', '.pyc', '.pyo', '.h', '.a', '.c', '.txt', '.html',
    '.xml', '.png', '.jpg', '.gif',
)

MAGIC = {
    '\xca\xfe\xba\xbe': 'MachO-universal',
    '\xce\xfa\xed\xfe': 'MachO-i386',
    '\xcf\xfa\xed\xfe': 'MachO-x86_64',
    '\xfe\xed\xfa\xce': 'MachO-ppc',
    '\xfe\xed\xfa\xcf': 'MachO-ppc64',
}


def is_macho(path):
    if path.endswith(NO_EXT) or islink(path) or not isfile(path):
        return False
    with open(path, 'rb') as fi:
        head = fi.read(4)
    return bool(head in MAGIC)


def get_arch(path):
    with open(path, 'rb') as fi:
        head = fi.read(4)
    try:
        return MAGIC[head][6:]
    except KeyError:
        return None


def otool(path):
    "thin wrapper around otool -L"
    lines = subprocess.check_output(['otool', '-L', path]).splitlines()
    assert lines[0] == path + ':'
    res = []
    for line in lines[1:]:
        assert line[0] == '\t'
        res.append(line.split()[0])
    return res


def install_name_change(path, cb_func):
    """
    change dynamic shared library install names of Mach-O binary `path`.

    `cb_func` is a callback function which called for each shared library name.
    It is called with `path` and the current shared library install name,
    and return the new name (or None if the name shoule be unchanged).
    """
    changes = []
    for link in otool(path):
        new_link = cb_func(path, link)
        if new_link:
            changes.append((link, new_link))

    for old, new in changes:
        args = ['install_name_tool', '-change', old, new, path]
        print(' '.join(args))
        subprocess.check_call(args)


def cp_arch(src_path, dst_path, arch='x86_64', ignore_wrong_arch=False):
    src_arch = get_arch(src_path)
    if src_arch == arch:
        shutil.copy2(src_path, dst_path)
    elif src_arch == 'universal':
        subprocess.check_call(['lipo', '-thin', arch,
                               '-output', dst_path, src_path])
    else:
        msg = "MachO type %r: %s" % (src_arch, src_path)
        if ignore_wrong_arch:
            sys.stdout.write('WARNING: ignoring %s\n' % msg)
        else:
            sys.exit('ERROR: %s' % msg)


def main_cp_arch():
    from optparse import OptionParser

    p = OptionParser(usage="usage: %prog [options] SRC [SRC ...] DST_DIR",
                     description="""\
Copy MachO binaries from SRC to DST_DIR, and use lipo to "thin" binaries (if
necessary), such that all binaries in DST_DIR are thin binaries of ARCH.""")

    p.add_option("--arch",
                 action="store",
                 default="x86_64",
                 help="target architecture, defaults to %default")

    p.add_option("--ignore-wrong-arch",
                 action="store_true",
                 help="ignore non-universal binaries of wrong ARCH")

    opts, args = p.parse_args()

    if len(args) < 2:
        p.error('two or more arguments expected, try -h')

    src_paths, dst_dir = args[:-1], args[-1]
    if not isdir(dst_dir):
        p.error('no such directory: %s' % dst_dir)

    for src_path in src_paths:
        cp_arch(src_path, join(dst_dir, basename(src_path)),
                arch=opts.arch, ignore_wrong_arch=opts.ignore_wrong_arch)


if __name__ == '__main__':
    main_cp_arch()
